/* config.h.  Generated automatically by configure.  */
/* $Header: /home/daffy/u0/vern/flex/RCS/conf.in,v 1.2 95/01/09 
12:11:51 vern Exp $ */

/* Define to empty if the keyword does not work.  */
/* #undef const */

/* Define to `unsigned' if <sys/types.h> doesn't define.  */
/* #undef size_t */

/* Define if you have the ANSI C header files.  */
#define STDC_HEADERS 1

/* Define if you have the <malloc.h> header file.  */
#define HAVE_MALLOC_H 1

/* Define if you have the <string.h> header file.  */
#define HAVE_STRING_H 1

/* Define if you have the <sys/types.h> header file.  */
#define HAVE_SYS_TYPES_H 1

/* Define if you have <alloca.h> and it should be used (not on 
Ultrix).  */
#define HAVE_ALLOCA_H 0

/* Define if platform-specific command line handling is necessary.  */
/* #undef NEED_ARGV_FIXUP */

/* Define if you use FAT file system, leave undefined for NTFS */
#undef SHORT_FILE_NAMES 
/* #define SHORT_FILE_NAMES 1 */
